@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsrasterlayer.h 6393 2007-01-04 08:13:13Z g_j_m $ */
+/* $Id: qgsrasterlayer.h 7698 2007-12-01 14:39:00Z jef $ */
 
 /** \file qgsrasterlayer.h
  *  \brief This class provides qgis with the ability to render raster datasets
@@ -61,7 +61,6 @@
  *     QFileInfo myFileInfo(myFileNameQString);
  *     QString myBaseNameQString = myFileInfo.baseName();
  *     QgsRasterLayer *myRasterLayer = new QgsRasterLayer(myFileNameQString, myBaseNameQString);
- *     myRasterLayer->initContextMenu(this); //prepare the right click pop up menu
  *
  *  In order to automate redrawing of a raster layer, you should like it to a map canvas like this :
  *  
@@ -170,12 +169,9 @@ The [type] part of the variable should be the type class of the variable written
 #include <QVector>
 #include <QList>
 
+#include "qgis.h"
 #include "qgspoint.h"
 #include "qgsmaplayer.h"
-#include "qgscolortable.h"
-#include "qgsrasterlayer.h"
-#include "qgsrasterbandstats.h"
-#include "qgsrasterpyramid.h"
 
 /*
  * 
@@ -190,21 +186,24 @@ The [type] part of the variable should be the type class of the variable written
  * END
  */
 
-
 #include <gdal_priv.h>
+
 //
 // Forward declarations
 //
+class QgsColorTable;
 class QgsRect;
+class QgsRasterBandStats;
+class QgsRasterPyramid;
 class QgsRasterLayerProperties;
+struct QgsRasterViewPort;
+class QImage;
+class QPixmap;
+
 class GDALDataset;
 class GDALRasterBand;
-class QImage;
 class QSlider;
 class QLibrary;
-//
-// Structs
-//
 
 
   
@@ -214,7 +213,7 @@ class QLibrary;
  *  onto the mapcanvas..
  */
 
-class QgsRasterLayer : public QgsMapLayer
+class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
 {
     Q_OBJECT
 public:
@@ -295,26 +294,30 @@ public:
     /** \brief Query gdal to find out the WKT projection string for this layer. This implements the virtual method of the same name defined in QgsMapLayer*/
     QString getProjectionWKT();
 
+    /** \brief Returns the number of raster units per each raster pixel. For rasters with world file, this is
+     normally the first row (without the sign) in that file */
+    double rasterUnitsPerPixel();
+
     /** \brief Draws a thumbnail of the rasterlayer into the supplied pixmap pointer */
      void drawThumbnail(QPixmap * theQPixmap);
 
     /** \brief Get an 8x8 pixmap of the colour palette. If the layer has no palette a white pixmap will be returned. */
      QPixmap getPaletteAsPixmap();
      
-    /** \brief This is called when the view on the rasterlayer needs to be refreshed (redrawn).  
-
-      \param drawingToEditingCanvas  Are we drawing to an editable canvas?
-                                  currently not used, but retain to be similar to
-                                  the QgsVectorLayer interface
+    /** \brief This is called when the view on the rasterlayer needs to be refreshed (redrawn).   
+         
+        \param drawingToEditingCanvas  Are we drawing to an editable canvas? 
+                                       currently not used, but retain to be similar to 
+                                       the QgsVectorLayer interface 
      */
     bool draw(QPainter * theQPainter,
-              QgsRect * theViewExtent,
+              QgsRect & theViewExtent, 
               QgsMapToPixel * theQgsMapToPixel,
+              QgsCoordinateTransform* ct,
               bool drawingToEditingCanvas);
 
     /** \brief This is an overloaded version of the above function that is called by both draw above and drawThumbnail */
-    void draw(QPainter * theQPainter,
-              QgsRasterViewPort * myRasterViewPort,
+    void draw(QPainter * theQPainter, QgsRasterViewPort * myRasterViewPort,
               QgsMapToPixel * theQgsMapToPixel = 0);
     
     //
@@ -362,10 +365,7 @@ public:
         stdDevsToPlotDouble = theDouble;
     };
     /** \brief Get the number of bands in this layer  */
-    const unsigned int getBandCount()
-    {
-        return rasterStatsVector.size();
-    };
+    const unsigned int getBandCount();
     /** \brief Get RasterBandStats for a band given its number (read only)  */
     const  QgsRasterBandStats getRasterBandStats(int);
     /** \brief  Check whether a given band number has stats associated with it */
@@ -383,15 +383,13 @@ public:
     const  QString getRasterBandName(int theBandNoInt);
     /** \brief Find out whether a given band exists.    */
     bool hasBand(QString const &  theBandName);
-    /** \brief accessor for transparency level.  */
-    unsigned int getTransparency();
     /** \brief Call any inline image manipulation filters */
     void filterLayer(QImage * theQImage);
     /** \brief Accessor for red band name (allows alternate mappings e.g. map blue as red colour). */
     QString getRedBandName()
     {
         return redBandNameQString;
-    };
+    }
     /** \brief Mutator for red band name (allows alternate mappings e.g. map blue as red colour). */
     void setRedBandName(QString const & theBandNameQString);
     // 
@@ -401,7 +399,7 @@ public:
     QString getGreenBandName()
     {
         return greenBandNameQString;
-    };
+    }
     /** \brief Mutator for green band name mapping.  */
     void setGreenBandName(QString const & theBandNameQString);
     //
@@ -411,7 +409,7 @@ public:
     QString getBlueBandName()
     {
         return blueBandNameQString;
-    };
+    }
     /** \brief Mutator for blue band name mapping.  */
     void setBlueBandName(QString const & theBandNameQString);
     //
@@ -421,7 +419,7 @@ public:
     QString getTransparentBandName()
     {
         return transparentBandNameQString;
-    };
+    }
     /** \brief Mutator for transparent band name mapping.  */
     void setTransparentBandName(QString const & theBandNameQString);
     //
@@ -431,7 +429,7 @@ public:
     QString getGrayBandName()
     {
         return grayBandNameQString;
-    };
+    }
     /** \brief Mutator for gray band name mapping.  */
     void setGrayBandName(QString const & theBandNameQString);
     // 
@@ -441,12 +439,12 @@ public:
     bool getShowDebugOverlayFlag()
     {
         return showDebugOverlayFlag;
-    };
+    }
     /** \brief Mutator for a flag that determines whether to show some debug info on the image.  */
     void setShowDebugOverlayFlag(bool theFlag)
     {
         showDebugOverlayFlag=theFlag;
-    };
+    }
     // 
     // Accessor and mutator for min and max red
     // 
@@ -458,7 +456,7 @@ public:
     double getMinRedDouble()
     {
         return minRedDouble;
-    };
+    }
     /** \brief Mutator for minimum clipping range for red.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -466,8 +464,9 @@ public:
      * that falls outside the clipping range.*/
     void setMinRedDouble(double theDouble)
     {
+        userDefinedColorMinMax = true;
         minRedDouble=theDouble;
-    };
+    }
     /** \brief Accessor for maximum clipping range for red.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -476,7 +475,7 @@ public:
     double getMaxRedDouble()
     {
         return maxRedDouble;
-    };
+    }
     /** \brief Mutator for maximum clipping range for red.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -484,8 +483,9 @@ public:
      * that falls outside the clipping range.*/
     void setMaxRedDouble(double theDouble)
     {
+        userDefinedColorMinMax = true;
         maxRedDouble=theDouble;
-    };
+    }
     // 
     // Accessor and mutator for min and max green
     // 
@@ -497,7 +497,7 @@ public:
     double getMinGreenDouble()
     {
         return minGreenDouble;
-    };
+    }
     /** \brief Mutator for minimum clipping range for green.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -505,8 +505,9 @@ public:
      * that falls outside the clipping range.*/
     void setMinGreenDouble(double theDouble)
     {
+        userDefinedColorMinMax = true;
         minGreenDouble=theDouble;
-    };
+    }
     /** \brief Accessor for maximum clipping range for green.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -515,7 +516,7 @@ public:
     double getMaxGreenDouble()
     {
         return maxGreenDouble;
-    };
+    }
     /** \brief Mutator for maximum clipping range for green.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -523,8 +524,9 @@ public:
      * that falls outside the clipping range.*/
     void setMaxGreenDouble(double theDouble)
     {
+        userDefinedColorMinMax = true;
         maxGreenDouble=theDouble;
-    };
+    }
     // 
     // Accessor and mutator for min and max blue
     // 
@@ -537,7 +539,7 @@ public:
     double getMinBlueDouble()
     {
         return minBlueDouble;
-    };
+    }
     /** \brief Mutator for minimum clipping range for blue.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -545,8 +547,9 @@ public:
      * that falls outside the clipping range.*/
     void setMinBlueDouble(double theDouble)
     {
+        userDefinedColorMinMax = true;
         minBlueDouble=theDouble;
-    };
+    }
     /** \brief Accessor for maximum clipping range for blue.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -555,7 +558,7 @@ public:
     double getMaxBlueDouble()
     {
         return maxBlueDouble;
-    };
+    }
     /** \brief Mutator for maximum clipping range for blue.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -563,8 +566,9 @@ public:
      * that falls outside the clipping range.*/
     void setMaxBlueDouble(double theDouble)
     {
+        userDefinedColorMinMax = true;
         maxBlueDouble=theDouble;
-    };
+    }
     // 
     // Accessor and mutator for min and max gray
     // 
@@ -576,7 +580,7 @@ public:
     double getMinGrayDouble()
     {
         return minGrayDouble;
-    };
+    }
     /** \brief Mutator for minimum clipping range for gray.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -584,8 +588,9 @@ public:
      * that falls outside the clipping range.*/
     void setMinGrayDouble(double theDouble)
     {
+        userDefinedGrayMinMax = true;
         minGrayDouble=theDouble;
-    };
+    }
     /** \brief Accessor for maximum clipping range for gray.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -594,7 +599,7 @@ public:
     double getMaxGrayDouble()
     {
         return maxGrayDouble;
-    };
+    }
     /** \brief Mutator for maximum clipping range for gray.
      *
      * The clipping range can have different interpretations - it can either be used to perform
@@ -602,12 +607,14 @@ public:
      * that falls outside the clipping range.*/
     void setMaxGrayDouble(double theDouble)
     {
+        userDefinedGrayMinMax = true;
         maxGrayDouble=theDouble;
-    };
+    }
     //
     /** \brief This enumerator describes the types of histogram scaling algorithms that can be used.  */
     enum COLOR_SCALING_ALGORITHM
     {
+        NO_STRETCH, //this should be the default color scaling algorithm, will allow for the display of images without calling QgsRasterBandStats unless needed
         STRETCH_TO_MINMAX, //linear histogram stretch
         STRETCH_AND_CLIP_TO_MINMAX,
         CLIP_TO_MINMAX
@@ -619,12 +626,12 @@ public:
     COLOR_SCALING_ALGORITHM getColorScalingAlgorithm()
     {
         return colorScalingAlgorithm;
-    };
+    }
     /** \brief Mutator for color scaling algorithm. */
     void setColorScalingAlgorithm(COLOR_SCALING_ALGORITHM theAlgorithm)
     {
         colorScalingAlgorithm=theAlgorithm;
-    };
+    }
     
     /** \brief This enumerator describes the types of histogram colour ramping that can be used.  */
     enum COLOR_RAMPING_TYPE
@@ -639,12 +646,12 @@ public:
     COLOR_RAMPING_TYPE getColorRampingType()
     {
         return colorRampingType;
-    };
+    }
     /** \brief Mutator for color scaling algorithm. */
     void setColorRampingType(COLOR_RAMPING_TYPE theRamping)
     {
         colorRampingType=theRamping;
-    };
+    }
     
     /** \brief This enumerator describes the different kinds of drawing we can do.  */
     enum DRAWING_STYLE
@@ -664,7 +671,7 @@ public:
     // Accessor and mutator for drawing style.
     //
     /** \brief Accessor for drawing style.  */
-    DRAWING_STYLE getDrawingStyle() {return drawingStyle;};
+    DRAWING_STYLE getDrawingStyle() {return drawingStyle;}
     /** \brief Returns a string representation of drawing style.
      *
      * Implementaed mainly for serialisation / deserialisation of settings to xml.
@@ -672,7 +679,7 @@ public:
      * */
     QString getDrawingStyleAsQString();
     /** \brief Mutator for drawing style.  */
-    void setDrawingStyle(DRAWING_STYLE const &  theDrawingStyle) {drawingStyle=theDrawingStyle;};
+    void setDrawingStyle(DRAWING_STYLE const &  theDrawingStyle) {drawingStyle=theDrawingStyle;}
     /** \brief Overloaded version of the above function for convenience when restoring from xml.
      *
      * Implementaed mainly for serialisation / deserialisation of settings to xml.
@@ -694,9 +701,9 @@ public:
     //accessor and for raster layer type (READ ONLY)
     //
     /** \brief  Accessor for raster layer type (which is a read only property) */
-    RASTER_LAYER_TYPE getRasterLayerType() { return rasterLayerType; };
+    RASTER_LAYER_TYPE getRasterLayerType() { return rasterLayerType; }
     /** \brief Accessor for hasPyramidsFlag (READ ONLY) */
-    bool getHasPyramidsFlag() {return hasPyramidsFlag;};
+    bool getHasPyramidsFlag() {return hasPyramidsFlag;}
      
     /** \brief Get a legend image for this layer.  */
     QPixmap getLegendQPixmap();
@@ -729,12 +736,6 @@ public:
      */
     virtual void setSubLayerVisibility(QString const & name, bool vis);
 
-    /** tailor the right-click context menu with raster layer only stuff 
-
-      @note called by QgsMapLayer::initContextMenu();
-     */
-    void initContextMenu_(QgisApp *);
-
     /** \brief Emit a signal asking for a repaint. (inherited from maplayer) */
     void triggerRepaint();
     /** \brief Obtain GDAL Metadata for this layer */
@@ -754,17 +755,21 @@ public:
     /** Return time stamp for given file name */
     static QDateTime lastModified ( QString const &  name );
 
-    /**Returns the path to an icon which characterises the type of layer*/
-    QString layerTypeIconPath();
-
-    /**Refresh the symbology part of the legend
-     by adding a child item to mLegendSymbologyGroupParent*/
-    void refreshLegend();
-
     /**Copies the symbology settings from another layer. Returns true in case of success*/
-    bool copySymbologySettings(const QgsMapLayer& other) {return false;} //todo
+    bool copySymbologySettings(const QgsMapLayer& other) {
+      //preventwarnings
+      if (other.type() < 0) 
+      {
+        return false;
+      }
+      return false;
+    } //todo
 
-    bool isSymbologyCompatible(const QgsMapLayer& other) const {return false;} //todo
+    bool isSymbologyCompatible(const QgsMapLayer& other) const
+    {
+      UNUSED(other);
+      return false;
+    } //todo
 
     /**
      * If an operation returns 0 (e.g. draw()), this function
@@ -794,11 +799,7 @@ public:
      */
     const QgsRasterDataProvider* getDataProvider() const;
 
-
-
 public slots:    
-    /** \brief Mutator for transparency level. Should be between 0 and 255 */
-    void setTransparency(unsigned int);
     /**
      * Convert this raster to another format
      */
@@ -808,18 +809,14 @@ public slots:
      **/
     void updateProgress(int,int);
 
-    /** sets whether this is in overview or not */
-    void inOverview( bool );
-
-    /** \brief Slot called when the popup menu transparency slider has been moved.*/
-    void popupTransparencySliderMoved(int);
-     
     /** \brief Create  gdal pyramid overviews  for this layer.
     * This will speed up performance at the expense of hard drive space.
     * Also, write access to the file is required. If no paramter is passed in
-    * it will default to nearest neighbor resampling. */
-    void buildPyramids(RasterPyramidList const &, 
-                       QString const &  theResamplingMethod="NEAREST");
+    * it will default to nearest neighbor resampling.
+    * \return null string on success, otherwise a string specifying error
+    */
+    QString buildPyramids(RasterPyramidList const &, 
+                          QString const &  theResamplingMethod="NEAREST");
     /** \brief Used at the moment by the above function but hopefully will later
     be useable by any operation that needs to notify the user of its progress. */
 /*
@@ -828,10 +825,6 @@ public slots:
                           void *theData);    
 */
 
-    /** \brief This method is called when the properties for this layer needs to be modified. 
-     * invokes an instance of the QgsRasterLayerProperties dialog box.*/
-    /* virtual */ void showLayerProperties();
-
   /** Populate the histogram vector for a given layer
   * @param theBandNoInt - which band to compute the histogram for
   * @param theBinCountInt - how many 'bins' to categorise the data into
@@ -978,16 +971,6 @@ private:
     /** \brief Update the layer if it is outdated */
     bool update ();
 
-    /**
-      set up the coordinate transform - in the case of raster this is mainly used to convert
-      the inverese projection of the map extents of the canvas when zooming in etc. so
-      that they match the coordinate system of this layer
-      if no other layers exist, set the output projection to be
-      the same as the input projection, otherwise set the output to the
-      project srs
-     */
-    void setupDestSrs();
-
     //
     // Private member vars
     //
@@ -1039,21 +1022,14 @@ private:
     double maxGrayDouble;
     /** \brief Whether this raster has overviews / pyramids or not */
     bool hasPyramidsFlag;
-    /** \brief These are two little icons used to indicate whether a 
-     * layer has pyramds bult or not. */
-    QPixmap mPyramidPixmap, mNoPyramidPixmap;
+    //Since QgsRasterBandStats deos not set the minRedDouble maxRedDouble etc., it is benificial to know if the user as set these values. Default = false
+    bool userDefinedColorMinMax;
+    bool userDefinedGrayMinMax;
     /** \brief This list holds a series of RasterPyramid structs
      * which store infomation for each potential pyramid level for this raster.*/
     RasterPyramidList mPyramidList;
-    //Transparency slider for popup menu
-    QSlider * mTransparencySlider; 
 
-    /* raster properties dialog 
 
-       @todo XXX should consider generalizing this
-    */
-    QgsRasterLayerProperties * mLayerProperties;
-    
 /*
  * 
  * New functions that will convert this class to a data provider interface
