@@ -1,9 +1,25 @@
+/***************************************************************************
+                             qgsspatialrefsys.h
+                       
+                             -------------------
+    begin                : 2007
+    copyright            : (C) 2007 by Gary E. Sherman
+    email                : sherman@mrcc.com
+***************************************************************************/
+
+/***************************************************************************
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ ***************************************************************************/
 #ifndef QGSSPATIALREFSYS_H
 #define QGSSPATIALREFSYS_H
 
 //Standard includes
 #include <ostream>
-#include <istream>
 
 //qt includes
 #include <QString>
@@ -11,65 +27,49 @@
 class QDomNode;
 class QDomDocument;
 
-#include <sqlite3.h>
+// forward declaration for sqlite3
+typedef struct sqlite3 sqlite3;
 
 //qgis includes
-#include <qgis.h>
+#include "qgis.h"
+
+class QgsSpatialRefSys;
+typedef void (*CUSTOM_SRS_VALIDATION)(QgsSpatialRefSys*);
 
-//gdal and ogr includes
-#include <ogr_api.h>
-#include <ogr_spatialref.h>
-#include <cpl_error.h>
 
 /*!
  * \class QgsSpatialRefSys
  * \brief Class for storing a spatial reference system (SRS)
  */ 
-class QgsSpatialRefSys
+class CORE_EXPORT QgsSpatialRefSys
 {
     public:
+      
+        enum SRS_TYPE {
+          QGIS_SRSID,
+          POSTGIS_SRID,
+          EPSG
+        };
+        
         //! Default constructor
-
         QgsSpatialRefSys();
+        
         /*! 
          * Constructs a SRS object from a WKT string
          * @param theWkt A String containing a valid Wkt def
          */
         explicit QgsSpatialRefSys(QString theWkt);
 
-        /*!
-         * Constructs a SRS object from the following component parts
-         *
-         * @param  long theSrsId The internal sqlite3 srs.db primary key for this srs 
-         * @param  QString the Description A textual description of the srs.
-         * @param  QString theProjectionAcronym The official proj4 acronym for the projection family
-         * @param  QString theEllipsoidAcronym The official proj4 acronym for the ellipoid
-         * @param  QString theProj4String Proj4 format specifies (excluding proj and ellips) that define this srs.
-         * @param  bool theGeoFlag Whether this is a geographic or projected coordinate system
-         * @param  long theSRID If available, the Postgis spatial_ref_sys identifier for this srs (defaults to 0)
-         * @param  long theEpsg If available the ESPG identifier for this srs (defaults to 0)
-         *
-         * @note THIS CTOR WILL PROABBLY BE REMOVED!!!!!!!!!!!!!!!!
+        /*! Use this constructor when you want to create a SRS object using 
+         *  a postgis SRID, an EPSG id or a QGIS SRS_ID.
+         * @param theId The ID no valid for the chosen coordinate system id type
+         * @param theType One of the types described in QgsSpatialRefSys::SRS_TYPE
          */
-        QgsSpatialRefSys(long theSrsId, 
-                QString theDescription, 
-                QString theProjectionAcronym, 
-                QString theEllipsoidAcronym, 
-                QString theProj4String,
-                long theSRID,
-                long theEpsg,
-                bool theGeoFlag);
+        QgsSpatialRefSys(const long theId, SRS_TYPE theType=POSTGIS_SRID);
 
-         enum SRS_TYPE {QGIS_SRSID,POSTGIS_SRID, EPSG};
-         /*! Use this constructor when you want to create a SRS object using 
-          *  a postgis SRID, an EPSG id or a QGIS SRS_ID.
-          * @param theId The ID no valid for the chosen coordinate system id type
-          * @param theType One of the types described in QgsSpatialRefSys::SRS_TYPE
-          */
-         QgsSpatialRefSys(const long theId, SRS_TYPE theType=POSTGIS_SRID);
 
-         // Assignment operator
-         QgsSpatialRefSys& operator=(const QgsSpatialRefSys& srs);
+        //! Assignment operator
+        QgsSpatialRefSys& operator=(const QgsSpatialRefSys& srs);
 
         // Misc helper functions -----------------------
 
@@ -195,16 +195,20 @@ class QgsSpatialRefSys
          *  Internally it will delegate to the equals method described below
          */
          bool operator==(const QgsSpatialRefSys &theSrs);
+        /*! Overloaded != operator used to compare to SRS's.
+          *  Returns opposite bool value to operator ==
+         */
+         bool operator!=(const QgsSpatialRefSys &theSrs);
         /*! Overloaded == operator used to compare to SRS's.
          *  Internally it will use OGR isSameSRS() or isSameGeoSRS() methods as appropriate.
          *  Additionally logic may also be applied if the result from the OGR methods
          *  is inconclusive.
          */
          bool equals(QString theProj4String);
-         /*! A helper to get an ogr representation of this srs
-          * @return OGRSpatialReference
+         /*! A helper to get an wkt representation of this srs
+          * @return string containing Wkt of the srs
           */
-         OGRSpatialReference toOgrSrs();
+         QString toWkt() const;
 
          /*! Restores state from the given DOM node.
          * @param theNode The node from which state will be restored
@@ -226,7 +230,7 @@ class QgsSpatialRefSys
          * @param theDom The document in which state will be stored
          * @return bool True on success, False on failure
          */
-          bool writeXML( QDomNode & theNode, QDomDocument & theDoc );
+          bool writeXML( QDomNode & theNode, QDomDocument & theDoc ) const;
 
          /** A static helper function to find out the proj4 string for a srsid
            * @param int theSrsId The srsid used for the lookup
@@ -234,6 +238,11 @@ class QgsSpatialRefSys
          */
          static QString getProj4FromSrsId(const int theSrsId);
 
+         /** Sets custom function to force valid SRS
+          *  QGIS uses implementation in QgisGui::customSrsValidation
+          */
+         static void setCustomSrsValidation(CUSTOM_SRS_VALIDATION f);
+         
         // Accessors -----------------------------------
 
          /*! Get the SrsId - if possible
@@ -318,6 +327,10 @@ class QgsSpatialRefSys
          * @param QString the acronym (must be a valid proj4 ellipsoid acronym)
          */
         void setEllipsoidAcronym(QString theEllipsoidAcronym);
+
+        /*! Print the description if debugging
+         */
+	void debugPrint();
     private:
         // Open SQLite db and show message if ccannot be opened
         // returns the same code as sqlite3_open
@@ -346,6 +359,8 @@ class QgsSpatialRefSys
 
         //! Work out the projection units and set the appropriate local variable
         void setMapUnits();
+        
+        static CUSTOM_SRS_VALIDATION mCustomSrsValidation;
 };
 
 
@@ -395,27 +410,4 @@ inline std::ostream& operator << (std::ostream& os, const QgsSpatialRefSys &r)
   return os << mySummary.toLocal8Bit().data() << std::endl;
 }
 
-
-/*
-//! Input stream operator
-inline std::istream& operator>> (std::istream& str, QgsSpatialRefSys& r)
-{
-  //std::cout << "FIXME FIXME" << __FILE__ << ":" << __LINE__ << std::endl;
-  std::string s;
-  str >> s;
-  
-  //QString srs = s.c_str();
-  // split the string into the parts to created the object
-  //QStringList parts = QStringList::split(QRegExp("\t"),srs);
-  // r.setSrid(parts[0]);
-  // r.setAuthName(parts[1]);
-  // r.setAuthSrid(parts[2]);
-  // r.setSrText(parts[3]);
-  // r.setProjText(parts[4]);
-  // r.setName(parts[5]);
-  //
-  return str;
-  
-} 
-*/
 #endif // QGSSPATIALREFSYS_H
