@@ -1,3 +1,17 @@
+#***************************************************************************
+#    ms_export.py
+#    --------------------------------------
+#   Date                 : Sun Sep 16 12:33:46 AKDT 2007
+#   Copyright            : (C) 2007 by Gary E. Sherman
+#   Email                : sherman at mrcc dot com
+#***************************************************************************
+#*                                                                         *
+#*   This program is free software; you can redistribute it and/or modify  *
+#*   it under the terms of the GNU General Public License as published by  *
+#*   the Free Software Foundation; either version 2 of the License, or     *
+#*   (at your option) any later version.                                   *
+#*                                                                         *
+#***************************************************************************/
 # This class exports a QGIS project file to a mapserver .map file.
 # All the work is done in the writeMapFile method. The msexport binary
 # presents a Qt based GUI that collects the needed information for this
@@ -6,7 +20,7 @@
 # CHANGES SHOULD NOT BE MADE TO THE writeMapFile METHOD UNLESS YOU
 # ARE CHANGING THE QgsMapserverExport CLASS AND YOU KNOW WHAT YOU ARE
 # DOING
-import sys, string
+import sys, string, os
 from xml.dom import minidom, Node
 
 # symbol map
@@ -31,6 +45,7 @@ class Qgis2Map:
     self.header = ''
     self.footer = ''
     self.symbolQueue = {}
+    
 
   # Set the options collected from the GUI
   def setOptions(self, units, image, mapname, width, height, template, header, footer):
@@ -54,49 +69,57 @@ class Qgis2Map:
     # write the general map and web settings
     print " --- python : map section "
     self.writeMapSection()
+    logmsg =  "Wrote map section\n"
     print " --- python : map section done"
     # write the projection section
     print " --- python : proj section "
     self.writeProjectionSection()
+    logmsg += "Wrote projection section\n"
     print " --- python : proj section done"
     # write the output format section
     print " --- python : outputformat section "
     self.writeOutputFormat()
+    logmsg += "Wrote output format section\n"
     print " --- python : outputformat section done"
     # write the legend section
     print " --- python : legend section"
     self.writeLegendSection()
+    logmsg += "Wrote legend section\n"
     print " --- python : legend section done"
 
     # write the WEB section
     print " --- python : web section "
     self.writeWebSection()
+    logmsg += "Wrote web section\n"
     print " --- python : web section done"
 
     # write the LAYER sections
     print " --- python : layer section "
     self.writeMapLayers()
+    logmsg += "Wrote map layers\n"
     print " --- python : layer section done"
 
     # write the symbol defs section
     # must happen after layers so we can build a symbol queue
     print " --- python : symbol section "
     self.writeSymbolSection()
+    logmsg += "Wrote symbol section\n"
     print " --- python : symbol section done"
 
     # END and close the map file
     self.outFile.write("END")
     self.outFile.close()
 
-    ret = "Writing the map file using " + self.project + " " + self.mapFile
-    return ret
+    logmsg += "Map file completed for " + os.path.basename(self.project) + "\n"
+    logmsg += "Map file saved as " + os.path.basename(self.mapFile) + "\n"
+    return logmsg
 
   # Write the general parts of the map section
   def writeMapSection(self):
     self.outFile.write("# Map file created from QGIS project file " + self.project + "\n")
     self.outFile.write("# Edit this file to customize for your map interface\n")
     self.outFile.write("MAP\n")
-    self.outFile.write("  NAME '" + self.mapName + "'\n")
+    self.outFile.write("  NAME " + self.mapName + "\n")
     self.outFile.write("  # Map image size\n")
     self.outFile.write("  SIZE " + self.width + " " + self.height + "\n")
     self.outFile.write("  UNITS " + self.units.lower() + "\n")
@@ -215,15 +238,28 @@ class Qgis2Map:
     pg['table'] = ''
     pg['geom'] = 'the_geom'
     
-    cmp = dataString.split(" ")
+    
+    whereCondition = dataString.split("sql")[1][1:]
+    cmp = dataString.split("sql")[0].split(" ")
+    
     for c in cmp:
       if c[:1] == "(":
         pg['geom'] = c[1:][:-1]
       else:
         kvp = c.split("=")
-        pg[kvp[0]] =  kvp[1]
-
-    return pg
+        if (len(kvp) >= 2):
+          pg[kvp[0]] =  kvp[1]
+    
+    connString = 'host=' + pg['host'] + " user=" + pg['user']
+    
+    if (len(pg['password'].replace("\'", "")) > 0):
+      connString += " password=" + pg['password'].replace("'", "")
+    
+    connString += " dbname=" + pg['dbname']
+     
+    dataString = pg['geom'] + " FROM " + pg['table'].replace("\"", "")
+    filterString = whereCondition.replace("\"", "")
+    return (connString, dataString, filterString)
 
        
   # Write the map layers
@@ -240,7 +276,7 @@ class Qgis2Map:
 
       self.outFile.write("  LAYER\n")
       # write the name of the layer
-      self.outFile.write("    NAME '" + lyr.getElementsByTagName("layername")[0].childNodes[0].nodeValue.encode('utf-8') + "'\n")
+      self.outFile.write("    NAME '" + lyr.getElementsByTagName("layername")[0].childNodes[0].nodeValue.encode('utf-8').replace("\"", "") + "'\n")
       if lyr.getAttribute("type").encode('utf-8') == 'vector':  
         self.outFile.write("    TYPE " + lyr.getAttribute("geometry").encode('utf-8').upper() + "\n")
       elif lyr.getAttribute("type").encode('utf-8') == 'raster':  
@@ -264,11 +300,11 @@ class Qgis2Map:
 
       if providerString == 'postgres':
         # it's a postgis layer
-        pg = self.parsePostgisConnection(dataString)
+        (pgConnString, sqlData, sqlFilter) = self.parsePostgisConnection(dataString)
         self.outFile.write("    CONNECTIONTYPE postgis\n")
-        self.outFile.write("    CONNECTION 'host=" + pg['host'] + " dbname=" + pg['dbname'] 
-                            + " password=" + pg['password'] + " user=" + pg['user'] + "'\n")
-        self.outFile.write("    DATA '" + pg['geom'] + " FROM " + pg['table'] + "'\n")
+        self.outFile.write("    CONNECTION '" + pgConnString + "'\n")
+        self.outFile.write("    DATA '" + sqlData + "'\n")
+        self.outFile.write("    FILTER '" + sqlFilter + "'\n")
 
       elif providerString == 'wms' and lyr.getAttribute("type").encode('utf-8').upper() == 'RASTER':
         # it's a WMS layer 
@@ -280,20 +316,23 @@ class Qgis2Map:
         wmsNames = []
         wmsStyles = []
         for wmsLayer in wmsSubLayers: 
-          wmsNames.append( wmsLayer.getElementsByTagName('name')[0].childNodes[0].nodeValue.encode('utf-8') )
+          wmsNames.append( wmsLayer.getElementsByTagName('name')[0].childNodes[0].nodeValue.encode('utf-8').replace("\"", "") )
           try: 
             wmsStyles.append( wmsLayer.getElementsByTagName('style')[0].childNodes[0].nodeValue.encode('utf-8') )
           except:
             wmsStyles.append( '' )
         # Create necesssary wms metadata
         format = rasterProp.getElementsByTagName('wmsFormat')[0].childNodes[0].nodeValue.encode('utf-8')
-        ct = lyr.getElementsByTagName('coordinatetransform')[0]
-        srs = ct.getElementsByTagName('sourcesrs')[0].getElementsByTagName('spatialrefsys')[0]
-        epsg = srs.getElementsByTagName('epsg')[0].childNodes[0].nodeValue.encode('utf-8')
         self.outFile.write("    METADATA\n")
         self.outFile.write("      'wms_name' '" + ','.join(wmsNames) + "'\n")
         self.outFile.write("      'wms_server_version' '1.1.1'\n")
-        self.outFile.write("      'wms_srs' 'EPSG:4326 EPSG:" + epsg + "'\n")
+        try:
+          ct = lyr.getElementsByTagName('coordinatetransform')[0]
+          srs = ct.getElementsByTagName('sourcesrs')[0].getElementsByTagName('spatialrefsys')[0]
+          epsg = srs.getElementsByTagName('epsg')[0].childNodes[0].nodeValue.encode('utf-8')
+          self.outFile.write("      'wms_srs' 'EPSG:4326 EPSG:" + epsg + "'\n")
+        except:
+	  pass
         self.outFile.write("      'wms_format' '" + format + "'\n")
         self.outFile.write("      'wms_style' '" + ','.join(wmsStyles) + "'\n")
         self.outFile.write("    END\n")
@@ -305,7 +344,7 @@ class Qgis2Map:
       # WMS settings for all layers
       self.outFile.write("    METADATA\n")
       self.outFile.write("      'wms_title' '" 
-           + lyr.getElementsByTagName("layername")[0].childNodes[0].nodeValue.encode('utf-8') + "'\n")
+           + lyr.getElementsByTagName("layername")[0].childNodes[0].nodeValue.encode('utf-8').replace("\"", "") + "'\n")
       self.outFile.write("    END\n")
 
       self.outFile.write("    STATUS DEFAULT\n")
@@ -315,7 +354,11 @@ class Qgis2Map:
       self.outFile.write("    TRANSPARENCY " + str(opacity) + "\n")
 
       self.outFile.write("    PROJECTION\n")
-      proj4Text = lyr.getElementsByTagName("proj4")[0].childNodes[0].nodeValue.encode('utf-8') 
+      # Get the destination srs for this layer and use it to create
+      # the projection section
+      destsrs = self.qgs.getElementsByTagName("destinationsrs")[0] 
+      proj4Text = destsrs.getElementsByTagName("proj4")[0].childNodes[0].nodeValue.encode('utf-8') 
+      # the proj4 text string needs to be reformatted to make mapserver happy
       self.outFile.write(self.formatProj4(proj4Text))
       self.outFile.write("    END\n")
       scaleDependent = lyr.getAttribute("scaleBasedVisibilityFlag").encode('utf-8')
@@ -331,9 +374,8 @@ class Qgis2Map:
       
       # Check for label field (ie LABELITEM) and label status
       try:
-        labelOn = lyr.getElementsByTagName("label")[0].childNodes[0].nodeValue.encode('utf-8')
-        labelNode = lyr.getElementsByTagName('labelattributes')[0]
-        labelField = labelNode.getElementsByTagName('label')[0].getAttribute('field').encode('utf-8')
+        labelOn    = lyr.getElementsByTagName(     "label")[0].childNodes[0].nodeValue.encode('utf-8')
+        labelField = lyr.getElementsByTagName("labelfield")[0].childNodes[0].nodeValue.encode('utf-8')
         if labelField != '' and labelField is not None and labelOn == "1":
           self.outFile.write("    LABELITEM '" + labelField + "'\n");
       except:
@@ -365,8 +407,7 @@ class Qgis2Map:
     self.outFile.write("    CLASS\n")
 
     self.outFile.write("       NAME '" 
-         + layerNode.getElementsByTagName("layername")[0].childNodes[0].nodeValue.encode('utf-8') 
-         + "'\n")
+         + layerNode.getElementsByTagName("layername")[0].childNodes[0].nodeValue.encode('utf-8').replace("\"", "") + "' \n")
 
     self.outFile.write("       STYLE\n")
     # use the point symbol map to lookup the mapserver symbol type
@@ -532,7 +573,11 @@ class Qgis2Map:
     for cls in classes:
       self.outFile.write("    CLASS\n")
 
-      lower = cls.getElementsByTagName('lowervalue')[0].childNodes[0].nodeValue.encode('utf-8')
+      try:
+        lower = cls.getElementsByTagName('lowervalue')[0].childNodes[0].nodeValue.encode('utf-8')
+      except IndexError:
+        # set to blank in the case where the field used for rendering has no value
+        lower = ""
 
       # If there's a label use it, otherwise autogenerate one
       try:
